################################################################################
## Library Imports
################################################################################

from extronlib import Version, event
from extronlib.device import ProcessorDevice, UIDevice
from extronlib.ui import Button, Level
from extronlib.system import MESet, Wait, Timer

print(Version())

################################################################################
## User Imports
################################################################################

import wyrestorm_mx_matrix_switchers_v1_0_0_0 as WyreStormModule

################################################################################
## Processor Definition
################################################################################

MyProcessor  = ProcessorDevice('MyProcessor')

################################################################################
## UI Definition
################################################################################

MyPanel = UIDevice('MyPanel')

################################################################################
## Module Instantiation
################################################################################

# TCP
MyModule = WyreStormModule.EthernetClass('192.168.1.85', Model="MX-1007-HYB")
#MyModule = WyreStormModule.EthernetClass('192.168.1.109', Model="MX-0804-EDC")

# RS-232
#MyModule = WyreStormModule.SerialClass(MyProcessor, 'COM2', Model="MX-1007-HYB")
#MyModule = WyreStormModule.SerialClass(MyProcessor, 'COM2', Model="MX-0804-EDC")

MyModule.enableDebug = True

################################################################################
## Constants / Global
################################################################################

MIN_INPUT_VOLUME                = -20
MAX_INPUT_VOLUME                = 20      
MAX_INPUT_VOLUME_MIC            = 40        

MIN_OUTPUT_VOLUME               = -100
MAX_OUTPUT_VOLUME               = 0

MAX_VIDEO_SWITCHING_INPUTS      = 10
MAX_VIDEO_SWITCHING_OUTPUTS     = 7
MAX_AUDIO_MIXER_INPUT_MIXES     = 3
MAX_AUDIO_MIXER_INPUT_SOURCES   = 15
MAX_AUDIO_MIXER_OUTPUT_MIXES    = 3
MAX_AUDIO_MIXER_OUTPUT_SOURCES  = 6
MAX_DISPLAYS                    = 6
MAX_MICS                        = 2
MAX_USB_HOSTS                   = 5
MAX_RELAYS                      = 2
MAX_SCENES                      = 6

BUTTON_STATES = ['Pressed', 'Released', 'Held', 'Repeated', 'Tapped']

POLL_TIMER = None
POLL_TIME = 120

AUDIO_MIXER_INPUT_MIXES = [
  "Mix 1",
  "Mix 2",
  "USB Mixer"
]

AUDIO_MIXER_INPUT_SOURCES = [
  "Out 1",
  "Out 2",
  "Out 3",
  "Out 4",
  "Out 5",
  "Out 6",
  "Out 7",
  "Dante In 1",
  "Dante In 2",
  "Dante In 3",
  "Dante In 4",
  "USB Host",
  "Mic In 1",
  "Mic In 2",
  "Line In"
]

AUDIO_MIXER_OUTPUT_MIXES = [
  "Mix 1",
  "Mix 2",
  "USB Mixer"
]

AUDIO_MIXER_OUTPUT_SOURCES = [
  "Line Out 1",
  "Line Out 2",
  "Amp Out",
  "Dante Out 1/2",
  "Dante Out 3/4",
  "NHD Out"
]

USB_HOST_NAMES = [
  "Type C",
  "USB Host 1",
  "USB Host 2",
  "In 9",
  "In 10"
]
 
################################################################################
## Button/Label Definitions
################################################################################

#-------------------------------------------------------------------------------
# Communicating / Initialized
#-------------------------------------------------------------------------------

btnCommunicating = Button(MyPanel, 1)
btnInitialized   = Button(MyPanel, 2)

#-------------------------------------------------------------------------------
# Menu/Popups
#-------------------------------------------------------------------------------

btnMainMenu = [
  Button(MyPanel, 11),    # Video Switching
  Button(MyPanel, 12),    # Audio Mixer Input
  Button(MyPanel, 13),    # Audio Mixer Output
  Button(MyPanel, 14),    # Display Power
  Button(MyPanel, 15),    # Phantom Power
  Button(MyPanel, 16),    # USB Host
  Button(MyPanel, 17),    # Relays
  Button(MyPanel, 18)     # Scenes
]

puMainMenuNames = [
  'Video_Switching',
  'Audio_Mixer_Input',
  'Audio_Mixer_Output',
  'Display_Power',
  'Phantom_Power',
  'USB_Host',
  'Relays',
  'Scenes'
]

btnMainMenuME = MESet(btnMainMenu)

#-------------------------------------------------------------------------------
# Top Menu
#-------------------------------------------------------------------------------

btnReconnect   = Button(MyPanel, 21)
btnEnableDebug = Button(MyPanel, 22)

#-------------------------------------------------------------------------------
# Video Switching
#-------------------------------------------------------------------------------

btnVideoSwitchingInput = [
  Button(MyPanel, 31),
  Button(MyPanel, 32),
  Button(MyPanel, 33),
  Button(MyPanel, 34),
  Button(MyPanel, 35),
  Button(MyPanel, 36),
  Button(MyPanel, 37),
  Button(MyPanel, 38),
  Button(MyPanel, 39),
  Button(MyPanel, 40),
  Button(MyPanel, 41)
]

btnVideoSwitchingInputME = MESet(btnVideoSwitchingInput)

btnVideoSwitchingOutput = [
  Button(MyPanel, 51),
  Button(MyPanel, 52),
  Button(MyPanel, 53),
  Button(MyPanel, 54),
  Button(MyPanel, 55),
  Button(MyPanel, 56),
  Button(MyPanel, 57),
  Button(MyPanel, 58)
]

btnVideoSwitchingOutputStatus = [
  Button(MyPanel, 61),
  Button(MyPanel, 62),
  Button(MyPanel, 63),
  Button(MyPanel, 64),
  Button(MyPanel, 65),
  Button(MyPanel, 66),
  Button(MyPanel, 67)
]

#-------------------------------------------------------------------------------
# Audio Mixer Input
#-------------------------------------------------------------------------------

btnAudioMixerInputMix = [
  Button(MyPanel, 71),
  Button(MyPanel, 72),
  Button(MyPanel, 73)
]

btnAudioMixerInputMixME = MESet(btnAudioMixerInputMix)

btnAudioMixerInputSource = [
  Button(MyPanel, 81),
  Button(MyPanel, 82),
  Button(MyPanel, 83),
  Button(MyPanel, 84),
  Button(MyPanel, 85),
  Button(MyPanel, 86),
  Button(MyPanel, 87),
  Button(MyPanel, 88),
  Button(MyPanel, 89),
  Button(MyPanel, 90),
  Button(MyPanel, 91),
  Button(MyPanel, 92),
  Button(MyPanel, 93),
  Button(MyPanel, 94),
  Button(MyPanel, 95)
]

btnAudioMixerInputSourceME = MESet(btnAudioMixerInputSource)

btnAudioMixerInputSourceEnable     = Button(MyPanel, 101)
btnAudioMixerInputSourceDisable    = Button(MyPanel, 102)
btnAudioMixerInputSourceStatus     = Button(MyPanel, 103)

btnAudioMixerInputVolumeUp         = Button(MyPanel, 121)
btnAudioMixerInputVolumeDown       = Button(MyPanel, 122)
btnAudioMixerInputVolumeMute       = Button(MyPanel, 123)
btnAudioMixerInputVolumeMuteStatus = Button(MyPanel, 133)
btnAudioMixerInputVolumeLevel      = Level(MyPanel, 141)

#-------------------------------------------------------------------------------
# Audio Mixer Output
#-------------------------------------------------------------------------------

btnAudioMixerOutputMix = [
  Button(MyPanel, 151),
  Button(MyPanel, 152),
  Button(MyPanel, 153)
]

btnAudioMixerOutputMixME = MESet(btnAudioMixerOutputMix)

btnAudioMixerOutputSource = [
  Button(MyPanel, 161),
  Button(MyPanel, 162),
  Button(MyPanel, 163),
  Button(MyPanel, 164),
  Button(MyPanel, 165),
  Button(MyPanel, 166)
]

btnAudioMixerOutputSourceME = MESet(btnAudioMixerOutputSource)

btnAudioMixerOutputSourceEnable     = Button(MyPanel, 171)
btnAudioMixerOutputSourceDisable    = Button(MyPanel, 172)
btnAudioMixerOutputSourceStatus     = Button(MyPanel, 173)

btnAudioMixerOutputVolumeUp         = Button(MyPanel, 181)
btnAudioMixerOutputVolumeDown       = Button(MyPanel, 182)
btnAudioMixerOutputVolumeMute       = Button(MyPanel, 183)
btnAudioMixerOutputVolumeMuteStatus = Button(MyPanel, 193)
btnAudioMixerOutputVolumeLevel      = Level(MyPanel, 201)

btnAudioMixerOutputVolumeLevel.SetRange(MIN_OUTPUT_VOLUME, MAX_OUTPUT_VOLUME)

#-------------------------------------------------------------------------------
# Display Power
#-------------------------------------------------------------------------------

btnDisplayPowerOn = [
  Button(MyPanel, 211),
  Button(MyPanel, 212),
  Button(MyPanel, 213),
  Button(MyPanel, 214),
  Button(MyPanel, 215),
  Button(MyPanel, 216),
  Button(MyPanel, 217)
]

btnDisplayPowerOff = [
  Button(MyPanel, 221),
  Button(MyPanel, 222),
  Button(MyPanel, 223),
  Button(MyPanel, 224),
  Button(MyPanel, 225),
  Button(MyPanel, 226),
  Button(MyPanel, 227)
]

#-------------------------------------------------------------------------------
# Phantom Power
#-------------------------------------------------------------------------------

btnPhantomPowerOn = [
  Button(MyPanel, 231),
  Button(MyPanel, 232)
]

btnPhantomPowerOff = [
  Button(MyPanel, 241),
  Button(MyPanel, 242)
]

btnPhantomPowerStatus = [
  Button(MyPanel, 251),
  Button(MyPanel, 252)
]

#-------------------------------------------------------------------------------
# USB Host
#-------------------------------------------------------------------------------

btnUsbHost = [
  Button(MyPanel, 261),
  Button(MyPanel, 262),
  Button(MyPanel, 263),
  Button(MyPanel, 264),
  Button(MyPanel, 265)
]

btnUsbHostStatus = [
  Button(MyPanel, 271),
  Button(MyPanel, 272),
  Button(MyPanel, 273),
  Button(MyPanel, 274),
  Button(MyPanel, 275)
]

#-------------------------------------------------------------------------------
# Relays
#-------------------------------------------------------------------------------

btnRelayOn = [
  Button(MyPanel, 281),
  Button(MyPanel, 282)
]

btnRelayOff = [
  Button(MyPanel, 291),
  Button(MyPanel, 292)
]

btnRelayStatus = [
  Button(MyPanel, 301),
  Button(MyPanel, 302)
]

#-------------------------------------------------------------------------------
# Scenes
#-------------------------------------------------------------------------------

btnSceneSave = [
  Button(MyPanel, 311),
  Button(MyPanel, 312),
  Button(MyPanel, 313),
  Button(MyPanel, 314),
  Button(MyPanel, 315),
  Button(MyPanel, 316)
]

btnSceneRecall = [
  Button(MyPanel, 321),
  Button(MyPanel, 322),
  Button(MyPanel, 323),
  Button(MyPanel, 324),
  Button(MyPanel, 325),
  Button(MyPanel, 326)
]

################################################################################
## Helpers
################################################################################

#-------------------------------------------------------------------------------
# Audio Mixer Input
#-------------------------------------------------------------------------------

def GetAudioMixerInputInfo():
  mixName = None
  sourceName = None

  currentMix = btnAudioMixerInputMixME.GetCurrent()
  currentSource = btnAudioMixerInputSourceME.GetCurrent()

  if currentMix is not None and currentSource is not None:
    mixIndex = btnAudioMixerInputMix.index(currentMix)
    sourceIndex = btnAudioMixerInputSource.index(currentSource)

    mixName = AUDIO_MIXER_INPUT_MIXES[mixIndex]
    sourceName = AUDIO_MIXER_INPUT_SOURCES[sourceIndex]

  return mixName, sourceName

#-------------------------------------------------------------------------------
# Audio Mixer Output
#-------------------------------------------------------------------------------

def GetAudioMixerOutputInfo():
  return GetAudioMixerOutputMixName(), GetAudioMixerOutputSourceName()


def GetAudioMixerOutputMixName():
  mixName = None

  currentMix = btnAudioMixerOutputMixME.GetCurrent()

  if currentMix is not None:
    mixIndex = btnAudioMixerOutputMix.index(currentMix)

    mixName = AUDIO_MIXER_OUTPUT_MIXES[mixIndex]

  return mixName


def GetAudioMixerOutputSourceName():
  sourceName = None

  currentSource = btnAudioMixerOutputSourceME.GetCurrent()

  if currentSource is not None:
    sourceIndex = btnAudioMixerOutputSource.index(currentSource)

    sourceName = AUDIO_MIXER_OUTPUT_SOURCES[sourceIndex]

  return sourceName

################################################################################
## Panel Events
################################################################################

#-------------------------------------------------------------------------------
# Menu/Popups
#-------------------------------------------------------------------------------

@event(btnMainMenu, BUTTON_STATES)
def OnMenuButtonPress(button, state):
  if state == 'Pressed':
    index = btnMainMenu.index(button)
    btnMainMenuME.SetCurrent(index)
    MyPanel.ShowPopup(puMainMenuNames[index])

#-------------------------------------------------------------------------------
# Top Menu
#-------------------------------------------------------------------------------

@event(btnReconnect, BUTTON_STATES)
def OnReconnectButtonPress(button, state):
  if state == 'Pressed':
    MyModule.Reinitialize()


@event(btnEnableDebug, BUTTON_STATES)
def OnEnableDebugButtonPress(button, state):
  if state == 'Pressed':
    MyModule.enableDebug = not MyModule.enableDebug
    btnEnableDebug.SetState(MyModule.enableDebug)

#-------------------------------------------------------------------------------
# Video Switching
#-------------------------------------------------------------------------------

@event(btnVideoSwitchingInput, BUTTON_STATES)
def OnVideoSwitchingInputButtonPress(button, state):
  if state == 'Pressed':
    index = btnVideoSwitchingInput.index(button)
    
    btnVideoSwitchingInputME.SetCurrent(index)
    UpdateVideoRouteOutputFeedback()


@event(btnVideoSwitchingOutput, BUTTON_STATES)
def OnVideoSwitchingOutputButtonPress(button, state):
  if state == 'Pressed':
    currentInput = btnVideoSwitchingInputME.GetCurrent()

    if currentInput is not None:
      outputIndex = btnVideoSwitchingOutput.index(button)
      inputIndex =  btnVideoSwitchingInput.index(currentInput)

      inputName = inputIndex < MAX_VIDEO_SWITCHING_INPUTS and "In {}".format(inputIndex + 1) or "None"
      outputName = outputIndex < MAX_VIDEO_SWITCHING_OUTPUTS and "Out {}".format(outputIndex + 1) or "All"

      MyModule.Set("VideoRoute", inputName, { "Output": outputName })

#-------------------------------------------------------------------------------
# Audio Mixer Input
#-------------------------------------------------------------------------------

@event(btnAudioMixerInputMix, BUTTON_STATES)
def OnAudioMixerInputMixButtonPress(button, state):
  if state == 'Pressed':
    index = btnAudioMixerInputMix.index(button)
    
    btnAudioMixerInputMixME.SetCurrent(index)
    UpdateAudioMixerInputFeedback()


@event(btnAudioMixerInputSource, BUTTON_STATES)
def OnAudioMixerInputSourceButtonPress(button, state):
  if state == 'Pressed':
    index = btnAudioMixerInputSource.index(button)
    
    btnAudioMixerInputSourceME.SetCurrent(index)

    sourceName = AUDIO_MIXER_INPUT_SOURCES[index]

    if sourceName == "Mic In 1" or sourceName == "Mic In 2":
      btnAudioMixerInputVolumeLevel.SetRange(MIN_INPUT_VOLUME, MAX_INPUT_VOLUME_MIC)
    else:
      btnAudioMixerInputVolumeLevel.SetRange(MIN_INPUT_VOLUME, MAX_INPUT_VOLUME)

    UpdateAudioMixerInputFeedback()


@event(btnAudioMixerInputSourceEnable, BUTTON_STATES)
def OnAudioMixerInputSourceEnableButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerInputInfo()

    if mixName and sourceName:
      MyModule.Set("AudioMixerInputSource", "On", { "Mix": mixName, "Source": sourceName })


@event(btnAudioMixerInputSourceDisable, BUTTON_STATES)
def OnAudioMixerInputSourceDisableButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerInputInfo()

    if mixName and sourceName:
      MyModule.Set("AudioMixerInputSource", "Off", { "Mix": mixName, "Source": sourceName })

      
@event(btnAudioMixerInputVolumeUp, BUTTON_STATES)
def OnAudioMixerInputVolumeUpButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerInputInfo()

    if mixName and sourceName:
      current = MyModule.ReadStatus("AudioMixerInputVolume", { "Mix": mixName, "Source": sourceName })
      
      if current is not None:
        max_ = (sourceName == "Mic In 1" or sourceName == "Mic In 2") and MAX_INPUT_VOLUME_MIC or MAX_INPUT_VOLUME
        new = current + 5

        if new > max_:
          new = max_

        MyModule.Set("AudioMixerInputVolume", new, { "Mix": mixName, "Source": sourceName })


@event(btnAudioMixerInputVolumeDown, BUTTON_STATES)
def OnAudioMixerInputVolumeDownButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerInputInfo()

    if mixName and sourceName:
      current = MyModule.ReadStatus("AudioMixerInputVolume", { "Mix": mixName, "Source": sourceName })
      
      if current is not None:
        new = current - 5

        if new < MIN_INPUT_VOLUME:
          new = MIN_INPUT_VOLUME

        MyModule.Set("AudioMixerInputVolume", new, { "Mix": mixName, "Source": sourceName })


@event(btnAudioMixerInputVolumeMute, BUTTON_STATES)
def OnAudioMixerInputVolumeMuteButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerInputInfo()

    if mixName and sourceName:
      MyModule.Set("AudioMixerInputMute", "Toggle", { "Mix": mixName, "Source": sourceName })

#-------------------------------------------------------------------------------
# Audio Mixer Output
#-------------------------------------------------------------------------------

@event(btnAudioMixerOutputMix, BUTTON_STATES)
def OnAudioMixerOutputMixButtonPress(button, state):
  if state == 'Pressed':
    index = btnAudioMixerOutputMix.index(button)
    
    btnAudioMixerOutputMixME.SetCurrent(index)
    UpdateAudioMixerOutputFeedback()


@event(btnAudioMixerOutputSource, BUTTON_STATES)
def OnAudioMixerOutputSourceButtonPress(button, state):
  if state == 'Pressed':
    index = btnAudioMixerOutputSource.index(button)
    
    btnAudioMixerOutputSourceME.SetCurrent(index)
    UpdateAudioMixerOutputFeedback()


@event(btnAudioMixerOutputSourceEnable, BUTTON_STATES)
def OnAudioMixerOutputSourceEnableButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerOutputInfo()

    if mixName and sourceName:
      MyModule.Set("AudioMixerOutputSource", "On", { "Mix": mixName, "Source": sourceName })


@event(btnAudioMixerOutputSourceDisable, BUTTON_STATES)
def OnAudioMixerOutputSourceDisableButtonPress(button, state):
  if state == 'Pressed':
    (mixName, sourceName) = GetAudioMixerOutputInfo()

    if mixName and sourceName:
      MyModule.Set("AudioMixerOutputSource", "Off", { "Mix": mixName, "Source": sourceName })

      
@event(btnAudioMixerOutputVolumeUp, BUTTON_STATES)
def OnAudioMixerOutputVolumeUpButtonPress(button, state):
  if state == 'Pressed':
    mixName = GetAudioMixerOutputMixName()
    
    if mixName:
      current = MyModule.ReadStatus("AudioMixerOutputVolume", { "Mix": mixName })
      
      if current is not None:
        new = current + 5

        if new > MAX_OUTPUT_VOLUME:
          new = MAX_OUTPUT_VOLUME

        MyModule.Set("AudioMixerOutputVolume", new, { "Mix": mixName })


@event(btnAudioMixerOutputVolumeDown, BUTTON_STATES)
def OnAudioMixerOutputVolumeDownButtonPress(button, state):
  if state == 'Pressed':
    mixName = GetAudioMixerOutputMixName()
    
    if mixName:
      current = MyModule.ReadStatus("AudioMixerOutputVolume", { "Mix": mixName })
      
      if current is not None:
        new = current - 5

        if new < MIN_OUTPUT_VOLUME:
          new = MIN_OUTPUT_VOLUME

        MyModule.Set("AudioMixerOutputVolume", new, { "Mix": mixName })


@event(btnAudioMixerOutputVolumeMute, BUTTON_STATES)
def OnAudioMixerOutputVolumeMuteButtonPress(button, state):
  if state == 'Pressed':
    mixName = GetAudioMixerOutputMixName()
    
    if mixName:
      MyModule.Set("AudioMixerOutputMute", "Toggle", { "Mix": mixName })

#-------------------------------------------------------------------------------
# Display Power
#-------------------------------------------------------------------------------

@event(btnDisplayPowerOn, BUTTON_STATES)
def OnDisplayPowerOnButtonPress(button, state):
  if state == 'Pressed':
    index = btnDisplayPowerOn.index(button)

    displayName = index < MAX_DISPLAYS and "Out {}".format(index + 1) or "All"
    
    MyModule.Set("DisplayPower", "On", { "Display": displayName })
    

@event(btnDisplayPowerOff, BUTTON_STATES)
def OnDisplayPowerOffButtonPress(button, state):
  if state == 'Pressed':
    index = btnDisplayPowerOff.index(button)

    displayName = index < MAX_DISPLAYS and "Out {}".format(index + 1) or "All"
    
    MyModule.Set("DisplayPower", "Off", { "Display": displayName })

#-------------------------------------------------------------------------------
# Phantom Power
#-------------------------------------------------------------------------------

@event(btnPhantomPowerOn, BUTTON_STATES)
def OnPhantomPowerOnButtonPress(button, state):
  if state == 'Pressed':
    index = btnPhantomPowerOn.index(button)

    if index < MAX_MICS:
      MyModule.Set("PhantomPower", "On", { "Mic": index + 1 })


@event(btnPhantomPowerOff, BUTTON_STATES)
def OnPhantomPowerOffButtonPress(button, state):
  if state == 'Pressed':
    index = btnPhantomPowerOff.index(button)

    if index < MAX_MICS:
      MyModule.Set("PhantomPower", "Off", { "Mic": index + 1 })

#-------------------------------------------------------------------------------
# USB Host
#-------------------------------------------------------------------------------

@event(btnUsbHost, BUTTON_STATES)
def OnUsbHostButtonPress(button, state):
  if state == 'Pressed':
    index = btnUsbHost.index(button)

    if index < MAX_USB_HOSTS:
      hostName = USB_HOST_NAMES[index]
    
      MyModule.Set("USBHost", hostName)

#-------------------------------------------------------------------------------
# Relays
#-------------------------------------------------------------------------------

@event(btnRelayOn, BUTTON_STATES)
def OnRelayOnButtonPress(button, state):
  if state == 'Pressed':
    index = btnRelayOn.index(button)

    if index < MAX_RELAYS:
      MyModule.Set("Relay", "On", { "Relay": index + 1 })


@event(btnRelayOff, BUTTON_STATES)
def OnRelayOffButtonPress(button, state):
  if state == 'Pressed':
    index = btnRelayOff.index(button)

    if index < MAX_RELAYS:
      MyModule.Set("Relay", "Off", { "Relay": index + 1 })

#-------------------------------------------------------------------------------
# Scenes
#-------------------------------------------------------------------------------

@event(btnSceneSave, BUTTON_STATES)
def OnSceneSaveButtonPress(button, state):
  if state == 'Pressed':
    index = btnSceneSave.index(button)

    if index < MAX_SCENES:
      MyModule.Set("Scene", "Save", { "Scene": index + 1 })


@event(btnSceneRecall, BUTTON_STATES)
def OnSceneRecallButtonPress(button, state):
  if state == 'Pressed':
    index = btnSceneRecall.index(button)

    if index < MAX_SCENES:
      MyModule.Set("Scene", "Recall", { "Scene": index + 1 })

################################################################################
## Feedback
################################################################################

#-------------------------------------------------------------------------------
# Video Switching
#-------------------------------------------------------------------------------

VideoStatus = {
 "Out 1": None,
 "Out 2": None,
 "Out 3": None,
 "Out 4": None,
 "Out 5": None,
 "Out 6": None,
 "Out 7": None
}

def UpdateVideoRouteOutputFeedback():
  currentInput = btnVideoSwitchingInputME.GetCurrent()

  if currentInput is not None:
    inputIndex =  btnVideoSwitchingInput.index(currentInput)
    inputName = inputIndex < MAX_VIDEO_SWITCHING_INPUTS and "In {}".format(inputIndex + 1) or 'None'

    for i in range(MAX_VIDEO_SWITCHING_OUTPUTS):
      outputName = "Out {}".format(i + 1)
      
      if outputName in VideoStatus:
        btnVideoSwitchingOutputStatus[i].SetState(VideoStatus[outputName] == inputName)

#-------------------------------------------------------------------------------
# Phantom Power
#-------------------------------------------------------------------------------

PhantomPowerStatus = [None] * MAX_MICS

def UpdatePhantomPowerFeedback():
  for i in range(MAX_MICS):
    btnPhantomPowerStatus[i].SetState(PhantomPowerStatus[i] == 'On')

#-------------------------------------------------------------------------------
# USB Host
#-------------------------------------------------------------------------------

USBHostStatus = None

def UpdateUSBHostFeedback():
  global USBHostStatus
  found = USBHostStatus in USB_HOST_NAMES

  for i in range(MAX_USB_HOSTS):
    btnUsbHostStatus[i].SetState(found and i == USB_HOST_NAMES.index(USBHostStatus))

#-------------------------------------------------------------------------------
# Relays
#-------------------------------------------------------------------------------

RelayStatus = [None] * MAX_RELAYS

def UpdateRelayFeedback():
  for i in range(MAX_RELAYS):
    btnRelayStatus[i].SetState(RelayStatus[i] == 'On')

#-------------------------------------------------------------------------------
# Audio Mixer Input
#-------------------------------------------------------------------------------

AudioMixerInputSource = {}
AudioMixerInputVolume = {}
AudioMixerInputMute = {}

for mix in AUDIO_MIXER_INPUT_MIXES:
  AudioMixerInputSource[mix] = {}
  AudioMixerInputVolume[mix] = {}
  AudioMixerInputMute[mix] = {}

  for source in AUDIO_MIXER_INPUT_SOURCES:
    AudioMixerInputSource[mix][source] = None
    AudioMixerInputVolume[mix][source] = None
    AudioMixerInputMute[mix][source] = None


def UpdateAudioMixerInputFeedback():
  (mixName, sourceName) = GetAudioMixerInputInfo()

  if mixName in AudioMixerInputSource:
    if sourceName in AudioMixerInputSource[mixName]:
      btnAudioMixerInputSourceStatus.SetState(AudioMixerInputSource[mixName][sourceName] == 'On')

    if sourceName in AudioMixerInputVolume[mixName]:
      level = AudioMixerInputVolume[mixName][sourceName]
      btnAudioMixerInputVolumeLevel.SetLevel(level is None and MIN_INPUT_VOLUME or level)

    if sourceName in AudioMixerInputMute[mixName]:
      btnAudioMixerInputVolumeMuteStatus.SetState(AudioMixerInputMute[mixName][sourceName] == 'On')

#-------------------------------------------------------------------------------
# Audio Mixer Output
#-------------------------------------------------------------------------------

AudioMixerOutputSource = {}
AudioMixerOutputVolume = {}
AudioMixerOutputMute = {}

for mix in AUDIO_MIXER_OUTPUT_MIXES:
  AudioMixerOutputSource[mix] = {}
  AudioMixerOutputVolume[mix] = None
  AudioMixerOutputMute[mix] = None

  for source in AUDIO_MIXER_OUTPUT_SOURCES:
    AudioMixerOutputSource[mix][source] = None


def UpdateAudioMixerOutputFeedback():
  (mixName, sourceName) = GetAudioMixerOutputInfo()

  if mixName in AudioMixerOutputVolume:
    level = AudioMixerOutputVolume[mixName]
    btnAudioMixerOutputVolumeLevel.SetLevel(level is None and MIN_OUTPUT_VOLUME or level)

  if mixName in AudioMixerOutputMute:
    btnAudioMixerOutputVolumeMuteStatus.SetState(AudioMixerOutputMute[mixName] == 'On')

  if sourceName:
    if mixName in AudioMixerOutputSource:
      if sourceName in AudioMixerOutputSource[mixName]:
        btnAudioMixerOutputSourceStatus.SetState(AudioMixerOutputSource[mixName][sourceName] == 'On')

################################################################################
## Module Status/Events
################################################################################

#-------------------------------------------------------------------------------
# All
#-------------------------------------------------------------------------------

def StatusRegister():
  MyModule.SubscribeStatus('Communicating', None, OnDeviceCommunicatingChange)
  MyModule.SubscribeStatus('Initialized', None, OnDeviceInitializedChange) 
  MyModule.SubscribeStatus('VideoRoute', None, OnDeviceVideoRouteChange) 
  MyModule.SubscribeStatus('PhantomPower', None, OnDevicePhantomPowerChange) 
  MyModule.SubscribeStatus('USBHost', None, OnDeviceUSBHostChange) 
  MyModule.SubscribeStatus('RelayControl', None, OnDeviceRelayChange) 
  MyModule.SubscribeStatus('AudioMixerInputSource', None, OnDeviceAudioMixerInputSourceChange) 
  MyModule.SubscribeStatus('AudioMixerInputVolume', None, OnDeviceAudioMixerInputVolumeChange) 
  MyModule.SubscribeStatus('AudioMixerInputMute', None, OnDeviceAudioMixerInputMuteChange)
  MyModule.SubscribeStatus('AudioMixerOutputSource', None, OnDeviceAudioMixerOutputSourceChange) 
  MyModule.SubscribeStatus('AudioMixerOutputVolume', None, OnDeviceAudioMixerOutputVolumeChange) 
  MyModule.SubscribeStatus('AudioMixerOutputMute', None, OnDeviceAudioMixerOutputMuteChange)

#-------------------------------------------------------------------------------
# Communicating / Initialized
#-------------------------------------------------------------------------------

def OnDeviceCommunicatingChange(command, value, qualifier):
  btnCommunicating.SetState(value == 'Communicating')

  if value != 'Communicating':
    ResetUI()
    StopPollTimer()


def OnDeviceInitializedChange(command, value, qualifier):
  btnInitialized.SetState(value == 'Initialized')
  
  if value == 'Initialized':
    RestartPollTimer()

#-------------------------------------------------------------------------------
# Video Switching
#-------------------------------------------------------------------------------

def OnDeviceVideoRouteChange(command, value, qualifier):
  VideoStatus[qualifier["Output"]] = value
  UpdateVideoRouteOutputFeedback()

#-------------------------------------------------------------------------------
# Phantom Power
#-------------------------------------------------------------------------------

def OnDevicePhantomPowerChange(command, value, qualifier):
  mic = qualifier and qualifier.get('Mic') or None

  if mic and mic > 0 and mic <= MAX_MICS:
    PhantomPowerStatus[mic - 1] = value
    UpdatePhantomPowerFeedback()

#-------------------------------------------------------------------------------
# USB Host
#-------------------------------------------------------------------------------

def OnDeviceUSBHostChange(command, value, qualifier):
  global USBHostStatus
  USBHostStatus = value
  UpdateUSBHostFeedback()

#-------------------------------------------------------------------------------
# Relays
#-------------------------------------------------------------------------------

def OnDeviceRelayChange(command, value, qualifier):
  relay = qualifier and qualifier.get('Relay') or None

  if relay and relay > 0 and relay <= MAX_RELAYS:
    RelayStatus[relay - 1] = value
    UpdateRelayFeedback()

#-------------------------------------------------------------------------------
# Audio Mixer Input
#-------------------------------------------------------------------------------

def OnDeviceAudioMixerInputSourceChange(command, value, qualifier):
  AudioMixerInputSource[qualifier["Mix"]][qualifier["Source"]] = value
  UpdateAudioMixerInputFeedback()


def OnDeviceAudioMixerInputVolumeChange(command, value, qualifier):
  AudioMixerInputVolume[qualifier["Mix"]][qualifier["Source"]] = value
  UpdateAudioMixerInputFeedback()


def OnDeviceAudioMixerInputMuteChange(command, value, qualifier):
  AudioMixerInputMute[qualifier["Mix"]][qualifier["Source"]] = value
  UpdateAudioMixerInputFeedback()

#-------------------------------------------------------------------------------
# Audio Mixer Output
#-------------------------------------------------------------------------------

def OnDeviceAudioMixerOutputSourceChange(command, value, qualifier):
  AudioMixerOutputSource[qualifier["Mix"]][qualifier["Source"]] = value
  UpdateAudioMixerOutputFeedback()


def OnDeviceAudioMixerOutputVolumeChange(command, value, qualifier):
  AudioMixerOutputVolume[qualifier["Mix"]] = value
  UpdateAudioMixerOutputFeedback()


def OnDeviceAudioMixerOutputMuteChange(command, value, qualifier):
  AudioMixerOutputMute[qualifier["Mix"]] = value
  UpdateAudioMixerOutputFeedback()

################################################################################
## Module Polling
################################################################################

def StartPollTimer():
  global POLL_TIMER
  global POLL_TIME
  if POLL_TIMER is None:
    POLL_TIMER = Wait(POLL_TIME, PollTimerEvent)


def StopPollTimer():        
  global POLL_TIMER
  if POLL_TIMER is not None:
    POLL_TIMER.Cancel()
    POLL_TIMER = None


def RestartPollTimer():
  StopPollTimer()
  StartPollTimer()


def PollTimerEvent(): 
  MyModule.Update('VideoRoute', {'Output': 'All'})

  for mix in AUDIO_MIXER_INPUT_MIXES:
    MyModule.Update('AudioMixerInputSource', { 'Mix': mix })

    for source in AUDIO_MIXER_INPUT_SOURCES:
      MyModule.Update('AudioMixerInputVolume', { 'Mix': mix, 'Source': source })
      MyModule.Update('AudioMixerInputMute', { 'Mix': mix, 'Source': source })
  
  for mix in AUDIO_MIXER_OUTPUT_MIXES:
    MyModule.Update('AudioMixerOutputSource', { 'Mix': mix })
    MyModule.Update('AudioMixerOutputVolume', { 'Mix': mix })
    MyModule.Update('AudioMixerOutputMute', { 'Mix': mix })

  for mic in range(MAX_MICS):
    MyModule.Update('PhantomPower', { 'Mic': mic + 1 })

  MyModule.Update('USBHost')

  for relay in range(MAX_RELAYS):
    MyModule.Update('RelayControl', { 'Relay': relay + 1 })
  
  RestartPollTimer()

################################################################################
## Reset UI
################################################################################

def ResetUI():
  # ------------------------- Communication/Initialization

  btnCommunicating.SetState(False)
  btnInitialized.SetState(False)

  # ------------------------- Menu/Popups

  btnMainMenuME.SetCurrent(0)
  MyPanel.ShowPopup(puMainMenuNames[0])

  # ------------------------- Video Switching
  btnVideoSwitchingInput[0].SetEnable(True)
  btnVideoSwitchingInputME.SetCurrent(None)
  
  for i in range(MAX_VIDEO_SWITCHING_OUTPUTS):
    btnVideoSwitchingOutputStatus[i].SetState(False)

  # ------------------------- Audio Mixer Input

  btnAudioMixerInputMixME.SetCurrent(None)
  btnAudioMixerInputSourceME.SetCurrent(None)
  btnAudioMixerInputSourceStatus.SetState(False)
  btnAudioMixerInputVolumeMuteStatus.SetState(False)
  btnAudioMixerInputVolumeLevel.SetLevel(MIN_INPUT_VOLUME)

  # ------------------------- Audio Mixer Output

  btnAudioMixerOutputMixME.SetCurrent(None)
  btnAudioMixerOutputSourceME.SetCurrent(None)
  btnAudioMixerOutputSourceStatus.SetState(False)
  btnAudioMixerOutputVolumeMuteStatus.SetState(False)
  btnAudioMixerOutputVolumeLevel.SetLevel(MIN_OUTPUT_VOLUME)

  # ------------------------- Phantom Power

  for i in range(MAX_MICS):
    btnPhantomPowerStatus[i].SetState(False)

  # ------------------------- USB Host

  for i in range(MAX_USB_HOSTS):
    btnUsbHostStatus[i].SetState(False)

  # ------------------------- Relays

  for i in range(MAX_RELAYS):
    btnRelayStatus[i].SetState(False)

################################################################################
## Initialize
################################################################################

def Initialize():
  ResetUI()
  
  btnEnableDebug.SetState(MyModule.enableDebug)
  
  StatusRegister()
  MyModule.Reinitialize()

################################################################################
## Main
################################################################################

Initialize()